/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.ListInstance;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.CompositeCell;
import com.google.gwt.cell.client.HasCell;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.AsyncHandler;
import com.google.gwt.user.cellview.client.IdentityColumn;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.AsyncDataProvider;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ImageActionCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.LabelCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.NullableDateCell;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.PListInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author Guillaume Porcher
 */
public class PListInstanceCellTable {

  public static final String COLUMN_LAST_UPDATE_DATE_TITLE = "Last Update Date";

  public static final String COLUMN_START_DATE_TITLE = "Start Date";

  public static final String COLUMN_STATE_TITLE = "State";

  private OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);


  private final HashMap<Column<?, ?>, String> sortableColumns = new HashMap<Column<?, ?>, String>();

  // Create a data provider.
  protected InstancesDataProvider dataProvider;

  private final PTabLayoutPanel tabLayoutPanel;

  private final String processId;
  private final String instanceId;
  private final String debugPrefix;


  public PListInstanceCellTable(
          final String processId, final PTabLayoutPanel ptabLayoutPanel, final ConsoleCellTable<MInstance> table,
          final String debugPrefix) {
    this(processId, null, ptabLayoutPanel, table, debugPrefix);
  }

  public PListInstanceCellTable(
          final String processId,
          final String instanceId,
          final PTabLayoutPanel ptabLayoutPanel,
          final ConsoleCellTable<MInstance> table,
          final String debugPrefix) {
    this.processId = processId;
    this.instanceId =instanceId;
    this.tabLayoutPanel = ptabLayoutPanel;

    this.debugPrefix = debugPrefix;

    table.ensureDebugId(debugPrefix);

    // Sorting is now provided by the pagination mechanism

    this.setColumns(table, instanceId == null);
    table.setRowCount(0, false);
    // Connect the table to the data provider.
    this.dataProvider = new InstancesDataProvider(processId, instanceId, table, sortableColumns,
            tabLayoutPanel);
    this.dataProvider.addDataDisplay(table);

    // Add a ColumnSortEvent.AsyncHandler to connect sorting to the
    // AsyncDataPRrovider.
    AsyncHandler columnSortHandler = new AsyncHandler(table);
    table.addColumnSortHandler(columnSortHandler);

  }

  private void setColumns(final ConsoleCellTable<MInstance> table, final boolean allowSorting) {
    /* UUID column */
    final Column<MInstance, String> uuidColumn = new Column<MInstance, String>(new LabelCell(this.debugPrefix)) {

      @Override
      public String getValue(final MInstance mprocess) {
        return mprocess.getInstanceId();
      }
    };
    // Add the columns.
    uuidColumn.setSortable(false);
    table.addColumn(uuidColumn, "UUID");

    /* Process UUID column */
    final Column<MInstance, String> processUUIDColumn = new Column<MInstance, String>(new LabelCell(this.debugPrefix)) {

      @Override
      public String getValue(final MInstance mprocess) {
        return mprocess.getProcessId();
      }
    };
    // Add the columns.
    processUUIDColumn.setSortable(false);
    table.addColumn(processUUIDColumn, "Process UUID");

    /* Name column */
    final Column<MInstance, String> processNameColumn = new Column<MInstance, String>(new LabelCell(this.debugPrefix)) {

      @Override
      public String getValue(final MInstance mprocess) {
        return mprocess.getProcessName();
      }
    };
    // Add the columns.
    processNameColumn.setSortable(false);
    table.addColumn(processNameColumn, "Process Name");

    /* State column */
    final Column<MInstance, String> stateColumn = new Column<MInstance, String>(new LabelCell(this.debugPrefix)) {

      @Override
      public String getValue(final MInstance mprocess) {
        return mprocess.getInstanceState();
      }
    };
    // Add the columns.
    if (allowSorting) {
      stateColumn.setSortable(true);
      sortableColumns.put(stateColumn, PListInstanceCellTable.COLUMN_STATE_TITLE);
    } else {
      stateColumn.setSortable(false);
    }
    table.addColumn(stateColumn, PListInstanceCellTable.COLUMN_STATE_TITLE);

    /* Start date column */
    final Column<MInstance, Date> deployDateColumn = new Column<MInstance, Date>(new NullableDateCell(this.debugPrefix)) {

      @Override
      public Date getValue(final MInstance mprocess) {
        return mprocess.getInstanceStartDate();
      }
    };
    // Add the columns.
    if (allowSorting) {
      deployDateColumn.setSortable(true);
      sortableColumns.put(deployDateColumn, PListInstanceCellTable.COLUMN_START_DATE_TITLE);
    } else {
      deployDateColumn.setSortable(false);
    }
    table.addColumn(deployDateColumn, PListInstanceCellTable.COLUMN_START_DATE_TITLE);

    /* Last update date column */
    final Column<MInstance, Date> undeployDateColumn = new Column<MInstance, Date>(new NullableDateCell(this.debugPrefix)) {

      @Override
      public Date getValue(final MInstance mprocess) {
        return mprocess.getInstanceLastUpdateDate();
      }
    };
    // Add the columns.
    if (allowSorting) {
      undeployDateColumn.setSortable(true);
      sortableColumns.put(undeployDateColumn, PListInstanceCellTable.COLUMN_LAST_UPDATE_DATE_TITLE);
    } else {
      undeployDateColumn.setSortable(false);
    }
    table.addColumn(undeployDateColumn, PListInstanceCellTable.COLUMN_LAST_UPDATE_DATE_TITLE);

    /* Actions column */
    final List<HasCell<MInstance, ?>> actionList = new ArrayList<HasCell<MInstance, ?>>();
    actionList.add(new IdentityColumn<MInstance>(this.getInstanceDetailsActionCell()));
    actionList.add(new IdentityColumn<MInstance>(this.getSuspendActionCell()));
    actionList.add(new IdentityColumn<MInstance>(this.getResumeActionCell()));
    actionList.add(new IdentityColumn<MInstance>(this.getExitActionCell()));
    actionList.add(new IdentityColumn<MInstance>(this.getRemoveActionCell()));

    final CompositeCell<MInstance> actionCells = new CompositeCell<MInstance>(actionList);
    final Column<MInstance, MInstance> actionsColumn = new IdentityColumn<MInstance>(actionCells);
    actionsColumn.setSortable(false);
    // Add the columns.
    table.addColumn(actionsColumn, "Actions");
  }

  private Cell<MInstance> getRemoveActionCell() {
    final Cell<MInstance> removeAction =
            new ImageActionCell<MInstance>(
                    ImageResources.INSTANCE.deleteIcon(), ImageResources.INSTANCE.deleteIconDisabled(), "remove", this.debugPrefix
                            + "-remove") {

              @Override
              public void execute(final MInstance object) {
                if (this.isActive(object)) {
                  PListInstanceCellTable.this.orchestramanagement.removeInstance(object.getInstanceId(), new AsyncCallback<Void>() {

                    public void onSuccess(final Void arg0) {
                      //As instance activity tab will be removed no need to update it with an event if
                      // PListInstanceCellTable.this.instanceId is not null.

                      ConsoleEventBus.getEventBus().fireEvent(
                              new ViewRefreshEvent(PListInstanceCellTable.this.processId, PTab.Type.LIST_INSTANCE, true));

                      // delete instance activity tab if exists
                      PListInstanceCellTable.this.tabLayoutPanel.removeTab(object.getInstanceId(), PTab.Type.LIST_INSTANCE_ACTIVITY);
                    }

                    public void onFailure(final Throwable arg0) {
                      if (arg0 instanceof OrchestraManagementException) {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(),
                                ((OrchestraManagementException) arg0).getDetails());
                      } else {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while remove instance !", arg0.getMessage());
                      }
                    }
                  });
                }
              }

              @Override
              public boolean isActive(final MInstance value) {
                return value.getInstanceState().equals("finished")
                        || value.getInstanceState().equals("exited")
                        || value.getInstanceState().equals("terminated");
              }
            };
    return removeAction;
  }

  private Cell<MInstance> getExitActionCell() {
    final Cell<MInstance> exitAction =
            new ImageActionCell<MInstance>(
                    ImageResources.INSTANCE.exit(), ImageResources.INSTANCE.exitDisabled(), "exit", this.debugPrefix + "-exit") {

              @Override
              public void execute(final MInstance object) {
                if (this.isActive(object)) {
                  PListInstanceCellTable.this.orchestramanagement.exitInstance(object.getInstanceId(), new AsyncCallback<Void>() {

                    public void onSuccess(final Void arg0) {
                      if (PListInstanceCellTable.this.instanceId == null) {
                      ConsoleEventBus.getEventBus().fireEvent(
                              new ViewRefreshEvent(PListInstanceCellTable.this.processId, PTab.Type.LIST_INSTANCE, true));
                      } else {
                        ConsoleEventBus.getEventBus().fireEvent( new ViewRefreshEvent(PListInstanceCellTable.this.instanceId,
                                PTab.Type.LIST_INSTANCE_ACTIVITY, true));
                      }
                    }

                    public void onFailure(final Throwable arg0) {
                      if (arg0 instanceof OrchestraManagementException) {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(),
                                ((OrchestraManagementException) arg0).getDetails());
                      } else {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while exit instance !", arg0.getMessage());
                      }
                    }
                  });
                }
              }

              @Override
              public boolean isActive(final MInstance value) {
                return value.getInstanceState().equals("running") || value.getInstanceState().equals("suspended");
              }
            };
    return exitAction;
  }

  private Cell<MInstance> getResumeActionCell() {
    final Cell<MInstance> resumeAction =
            new ImageActionCell<MInstance>(
                    ImageResources.INSTANCE.resume(), ImageResources.INSTANCE.resumeDisabled(), "resume", this.debugPrefix + "-resume") {

              @Override
              public void execute(final MInstance object) {
                if (this.isActive(object)) {
                  PListInstanceCellTable.this.orchestramanagement.resumeInstance(object.getInstanceId(), new AsyncCallback<Void>() {

                    public void onSuccess(final Void arg0) {
                      if (PListInstanceCellTable.this.instanceId == null) {
                      ConsoleEventBus.getEventBus().fireEvent(
                              new ViewRefreshEvent(PListInstanceCellTable.this.processId, PTab.Type.LIST_INSTANCE, true));
                      } else {
                        ConsoleEventBus.getEventBus().fireEvent( new ViewRefreshEvent(PListInstanceCellTable.this.instanceId,
                                PTab.Type.LIST_INSTANCE_ACTIVITY, true));
                      }
                    }

                    public void onFailure(final Throwable arg0) {
                      if (arg0 instanceof OrchestraManagementException) {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(),
                                ((OrchestraManagementException) arg0).getDetails());
                      } else {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while resume instance !", arg0.getMessage());
                      }
                    }
                  });
                }
              }

              @Override
              public boolean isActive(final MInstance value) {
                return value.getInstanceState().equals("suspended");
              }
            };
    return resumeAction;
  }

  private Cell<MInstance> getSuspendActionCell() {
    final Cell<MInstance> suspendAction =
            new ImageActionCell<MInstance>(
                    ImageResources.INSTANCE.suspend(), ImageResources.INSTANCE.suspendDisabled(), "suspend", this.debugPrefix
                            + "-suspend") {

              @Override
              public void execute(final MInstance object) {
                if (this.isActive(object)) {
                  PListInstanceCellTable.this.orchestramanagement.suspendInstance(object.getInstanceId(), new AsyncCallback<Void>() {

                    public void onSuccess(final Void arg0) {
                      if (PListInstanceCellTable.this.instanceId == null) {
                      ConsoleEventBus.getEventBus().fireEvent(
                              new ViewRefreshEvent(PListInstanceCellTable.this.processId, PTab.Type.LIST_INSTANCE, true));
                      } else {
                        ConsoleEventBus.getEventBus().fireEvent( new ViewRefreshEvent(PListInstanceCellTable.this.instanceId,
                                PTab.Type.LIST_INSTANCE_ACTIVITY, true));
                      }
                    }

                    public void onFailure(final Throwable arg0) {
                      ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
                      if (arg0 instanceof OrchestraManagementException) {
                        PPopup.launch(
                                PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException) arg0).getDetails());
                      } else {
                        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while suspend instance !", arg0.getMessage());
                      }
                    }
                  });
                }
              }

              @Override
              public boolean isActive(final MInstance value) {
                return value.getInstanceState().equals("running");
              }
            };
    return suspendAction;
  }

  private Cell<MInstance> getInstanceDetailsActionCell() {
    final Cell<MInstance> instanceDetailsAction = new ImageActionCell<MInstance>(ImageResources.INSTANCE.activities(),
            ImageResources.INSTANCE.activitiesDisabled(), "activities", this.debugPrefix + "-activities") {

      @Override
      public void execute(final MInstance object) {
        // create the presenter of list process activity
        new PListInstanceActivity(PListInstanceCellTable.this.tabLayoutPanel, object.getInstanceId(),
                object.getProcessName() + " " + object.getInstanceId());
      }

      @Override
      public boolean isActive(final MInstance value) {
        return true;
      }
    };
    return instanceDetailsAction;
  }

}
